/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

/* push.c */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "motifR.h"
#include "Headers.h"

Widgets* widget_ptr = new Widgets;


/************************************* contents *****************************/
/*
 *	CreatePushbutton -- creates a pushbutton widget
 *	CreatePulldownMenu -- creates a pulldown menu but not menuitems
 *	CreateHelpPulldownMenu -- creates a help pulldown menu
 *	CreateDrawingArea -- creates a drawing area and adds callbacks
 *	exitCB	-- a generic exit callback routine
 *	genericCB -- a generic CB routine that simply prints out content
 *			of client data
 */
/****************************************************************************/

/****************************************************************************/
/* creates a pushbutton widget as a child of the parent window
 *  that calls procedure callback when activated
 */
/****************************************************************************/

Widget CreatePushbutton(Widget parent,	/* parent */
	char* name,			/* name of pushbutton widget */
	XtCallbackProc callback,	/* callback function to call */
	XtPointer client_data		/* args to pass to callback procedure */
	)

{
	Widget		push;
	Cardinal	n;
	Arg		args[20];

	/* create pushbutton widget called push */

	n = 0;
	XtSetArg(args[n], XmNshadowThickness, 6); n++;
	push = XmCreatePushButton( parent,	/* parent of widget */
				 name,		/* name of widget */
				 args, n	/* resources */
				 );

	/* set up a callback routine for the pushbutton widget */
	/* this registers a function as a callback for this widget */

	/* XmNactivateCallback seems to refer to a callback
	 * anytime the user activates the widget */

	XtAddCallback(push,		/* widget */
		XmNactivateCallback,	/* which callback */
		(XtCallbackProc) callback,/* callback function */
		(XtPointer) client_data	/* extra data to pass */
		);

	/* Manage the pushbutton widget */

	XtManageChild(push);

	return(push);

} /* CreatePushbutton */



/************************************************************************/
/*									
 *		creates a pull-down menu			
 */
/************************************************************************/

Widget CreatePulldownMenu(Widget parent,	/* parent is usually menubar */
			  char* name		/* name for pulldown menu */
			 )

{

	Cardinal n;
	Widget	menu;		/* pulldown menu widget -- is returned */
	Widget  cascade;	/* cascade button widget */
	Arg	args[20];	

	/* create menu -- do not manage */

	n = 0;

#ifdef XmNtearOffModel		/* Engage tear off menu if supported */

	XtSetArg(args[n], XmNtearOffModel, XmTEAR_OFF_ENABLED); n++;

#endif /* XmNtearOffModel */

	/* create pulldown menu but don't manage */

	menu = XmCreatePulldownMenu(parent, name, args, n);

	/* create cascade buttons for menu */

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, menu); n++;

	cascade = XmCreateCascadeButton(parent, name, args, n);

	XtManageChild(cascade);

	/* this is the part that makes it a help menu */
	/*  tells parent that this is a help menu */
	/*  essentially this just puts the menu at the right */

/*	XtVaSetValues(parent, XmNmenuHelpWidget, cascade, NULL);	*/

	return menu;
}



/************************************************************************/
/*									
 *		creates a help pull-down menu				
 *									
 */
/************************************************************************/

Widget CreateHelpMenu(Widget parent,	/* parent is usually menubar */
			  char* name	/* name for help menu */
			 )
{

	Cardinal n;
	Widget	menu;		/* pulldown menu widget -- is returned */
	Widget  cascade;	/* cascade button widget */
	Arg	args[20];	

	/* create menu -- do not manage */

	n = 0;

#ifdef XmNtearOffModel		/* Engage tear off menu if supported */

	XtSetArg(args[n], XmNtearOffModel, XmTEAR_OFF_ENABLED); n++;

#endif /* XmNtearOffModel */

	/* create pulldown menu but don't manage */

	menu = XmCreatePulldownMenu(parent, name, args, n);

	/* create cascade buttons for menu */

	n = 0;
	XtSetArg(args[n], XmNsubMenuId, menu); n++;

	cascade = XmCreateCascadeButton(parent, name, args, n);

	XtManageChild(cascade);

	/* this is the part that makes it a help menu */
	/*  tells parent that this is a help menu */
	/*  essentially this just puts the menu at the right */

	XtVaSetValues(parent, XmNmenuHelpWidget, cascade, NULL);

	return menu;
}


/************************ CreateDrawingArea *********************************/
/*
 *	creates a drawing area for graphics
 */
/****************************************************************************/

	
Widget CreateDrawingArea(Widget parent,		/* parent of widget */
		char* name,			/* name for help menu */
		XtCallbackProc  callback,	/* callback function to call */
		XtPointer client_data	/* args to pass to callback procedure */
	)
{
	Cardinal	n;
	Arg		args[20];
	Widget		drawarea;

	/* set up resources */

	n = 0;

	client_data = client_data;
	

	/* allow resizing */

	XtSetArg(args[n], XmNresizePolicy, XmRESIZE_ANY); n++;\

	/* set initial width and height of drawing area */

	/* initial width and height is set in resource file */

	drawarea = XmCreateDrawingArea(parent, name, args, n);

	/* add callbacks -- assume all get same callback -- essentially
	 * 	a redraw for each -- you can treat the different cases
	 *	in the callback procedure */

	XtAddCallback(drawarea, XmNexposeCallback, (XtCallbackProc) callback,
		(XtPointer) client_data);

	XtAddCallback(drawarea, XmNinputCallback, (XtCallbackProc) callback,
		(XtPointer) client_data);

	XtAddCallback(drawarea, XmNresizeCallback, (XtCallbackProc) callback,
		(XtPointer) client_data);

	XtManageChild(drawarea);

	return	drawarea;
}


/****************** A variety of callbacks procedures *********************/
 
/***************************** exitCB **************************************/
/*
 * exitCB() is a callback for the pushbutton widget
 * which is created below.  exitCB() calls exit
 * to terminate the program
 */
/***************************************************************************/

void exitCB(Widget widget,	/* widget == a user-interface component
				 * such as scroll bars, push buttons, dialogs
				 * and so forth */	
	XtPointer client_data,	/* pointer to any extra data you wish
				 * to pass to call back function 
				 */
	XtPointer call_data	/* data from the Motif or Xt toolkit */
	)

				/* widget == a user-interface component
				 * such as scroll bars, push buttons, dialogs
				 * and so forth */
{	
	client_data = client_data;
	call_data = call_data;
	widget = widget;

	/* exitCB */
	exit(0);
}



/************************  genericCB  ***************************************/
/*	a generic call back routine which prints out client_data
 *	if it is not empty -- this serves as a check for program
 */
/****************************************************************************/

void genericCB(Widget widget, XtPointer client_data, XtPointer call_data)

{
	widget = widget;
	call_data = call_data;

	if( client_data != NULL ){
	  printf("Menu callback for [%s]\n", (char*) client_data);
	}

} /* end genericCB */
 

/************************ drawing callback ******************************/
/*	right now all it does is updates the widget			
 *		this is probably not necessary since it is probably
 *		already updated when called anyway
 */

void drawCB(Widget widget, XtPointer client_data, XtPointer call_data)
{
/*	GC*	gc_ptr;					*/
	XmDrawingAreaCallbackStruct*	ptr;
	Dimension	width, height;
	char		message[80];

/*	gc_ptr = (GC*) client_data;		*/
	widget = widget;
	client_data = client_data;
	call_data = call_data;
        widget_ptr = ReturnClassAddress();


	/* get information from the call using call_data which is returned 
	 *	by the XtAppNextEvent routine */

	ptr = (XmDrawingAreaCallbackStruct*) call_data;

	if( ptr == NULL ){
		return;		/* if call_data was empty */
	}

        switch(ptr->reason){
           case XmCR_INPUT:	/* window was clicked on */
        	break; 		
	   case XmCR_RESIZE:	/* window was resized */	
		XtVaGetValues(widget, XmNwidth, &width, XmNheight, 
			&height, NULL);
                sprintf(message, "RESIZING WIDGET TO: %d %d \n",
                        (int)width, (int)height);
                XmTextReplace((*widget_ptr).dgr_message_window, (XmTextPosition) 0,
                        XmTextGetLastPosition((*widget_ptr).dgr_message_window),
                        message);
		break;
	    case XmCR_EXPOSE:	/* window was hidden but now exposed */
		break;
        }

	return;
}

/*************************************************************************/
/*
 *	creates a text field with text label and float number
 */
/*************************************************************************/

Widget createTableFloat(Widget parent,	/* parent widget */
	 	char* name,  		/* name of text widget */
		char* text,		/* text label for field */
		float time		/* number in field */
	)
{
	Widget	widget;
	XmTextPosition	lastposition;

	/* create widget */

	widget = XtVaCreateManagedWidget(text, xmLabelWidgetClass,
			parent, NULL);

	widget = XtVaCreateManagedWidget(name, xmTextWidgetClass, parent,
		 	XmNeditMode, XmSINGLE_LINE_EDIT, NULL);

	/* insert label text */

/*

	XmTextInsert(widget, (XmTextPosition) 0, text);
	lastposition = XmTextGetLastPosition(widget);
*/
	lastposition = 0;

	pasteFloatToWidget(time, 5, widget, lastposition);

	return widget;
}


/****************************************************************************/
/*	a routine that replaces a section of the string in widget
 *		from firstPosition to end of string in widget
 *		by a floating point number after its been converted
 *	note the widget must be a text widget
 *
 *	note: gcvt attempts to get digits number of significant digits
 */

char*	pasteFloatToWidget(
		float float_number,	/* real to paste */
		int   digits,		/* number of significant digits */
		Widget widget,		/* destination widget */
		XmTextPosition firstPosition	/* first position to place */
		)
{
	char	pasteString[64];

	gcvt((double)float_number, digits, pasteString);
	XmTextReplace(widget, firstPosition,
		(XmTextPosition) XmTextGetLastPosition(widget),
			pasteString);

	return pasteString;
}


/************************** callback for prompt *************************/

void promptCB(Widget widget, XtPointer client_data, XtPointer call_data)
{
	XmSelectionBoxCallbackStruct*	ptr;
	float*		floatptr;
	char*		string;

	/* get string from prompt box */

	widget = widget;
	ptr = (XmSelectionBoxCallbackStruct*) call_data;
	XmStringGetLtoR(ptr->value, XmSTRING_DEFAULT_CHARSET, &string);

	/* put new value into box */

	floatptr = (float*) client_data;
	*floatptr = (float)atof(string);

	XtFree(string);

}

/************************** callback for cancel *************************/
/*	erases entry to prompt dialogue and replaces with previous value
 */
/************************************************************************/

void cancel_prompt_CB(Widget widget, XtPointer client_data, XtPointer call_data)
{
	float*		floatptr;
	XmString	xmstring;

	/* put old value of float back into box */
	call_data = call_data;
	floatptr = (float*) client_data; 
	xmstring = makeXmStringFromFloat(*floatptr);
	XtVaSetValues(widget, XmNtextString, xmstring, NULL);
	XmStringFree(xmstring);

}

XmString makeXmStringFromFloat(float fnumber)
{
	char		string[64];
	XmString	xmstring;
	
        gcvt((double)fnumber, 8, string);
        xmstring = XmStringCreateSimple(string);
        XtFree(string);
	return xmstring;

}


/**************** simply a callback which manages a widget ****************/


void manageCB(Widget widget, XtPointer client_data, XtPointer call_data)

{
	Widget*	widgptr;

	widget = widget;
	client_data = client_data;
	call_data = call_data;
	
	widgptr = (Widget*) client_data;
	XtManageChild(*widgptr);

}


/*****************************************************************/

/* creates an unmanaged prompt dialogue with slection text
 *	equal to text and with default value given by fnumber
 *	pasted into the prompt -- it also sets up the OK
 *	callback procedure and the client_data for the callback
 */

Widget CreatePromptDialogWithFloat(
	Widget parent,
	char*	name,
	char*	text,
	float*	fptr,
	XtCallbackProc callback,
	XtPointer client_data
	)
{
	Widget	widget;
	XmString	xmstring;
	Arg	args[5];

	/* make the prompt widget and make title */

	widget = XmCreatePromptDialog(parent, name, args, (Cardinal) 0);
	xmstring = XmStringCreateSimple(text);
	XtVaSetValues(widget, XmNselectionLabelString,
		xmstring, NULL);

	/* put the present value in the dialog box 
	 *   you have to convert this to xmstring before
	 *   pasting into the prompt box
	 */

	xmstring = makeXmStringFromFloat(*fptr);
	XtVaSetValues(widget, XmNtextString, xmstring, NULL);

	/* set up the prompt callback */

	XtAddCallback(widget, XmNokCallback, (XtCallbackProc) callback,
		(XtPointer) client_data);

	XtAddCallback(widget, XmNcancelCallback,
		(XtCallbackProc) cancel_prompt_CB, (XtPointer) client_data);

	return widget;
}




/**** callbacks on return press in widget gets a new float widget *******/

void newfloatCB(Widget widget,  XtPointer client_data, XtPointer call_data)

{
	char*	string;
	float*	floatptr;

	widget = widget;
	call_data = call_data;
	floatptr = (float*) client_data;

	string = XmTextGetString(widget);
	*floatptr = (float)atof(string);

	XtFree(string);
}		



/************************  continueCB ***************************************/
/*
 *	callback function for continue button
 *		adds to CONTINUE_TIME to endtime and continues run
 */
/****************************************************************************/


void continueCB(Widget widget, XtPointer client_data, XtPointer call_data)

{
	FptrWptr* ptr;		/* pointer to data structure */
	float*	end_ptr;	/* pointer to end time */
	Widget*	text_widget;	/* widget containing the endtime window */
 	char    message[80];    /* message string for message window */

	widget = widget;
	call_data = call_data;
        if( client_data == NULL ) return;

	ptr = (FptrWptr*) client_data;

	end_ptr = ptr->floatptr;	/* use -> as ptr is pointer to struct */
	text_widget = ptr->widgptr;

	*end_ptr = (float)floor(*end_ptr);
			/* eliminates the noninteger part of the endtime */
        *end_ptr = *end_ptr + CONTINUE_TIME;

        sprintf(message, "adding %i to ending time: %f",
                CONTINUE_TIME,  *end_ptr);
        XmTextReplace((*widget_ptr).dgr_message_window, (XmTextPosition) 0,
                XmTextGetLastPosition((*widget_ptr).dgr_message_window),
                message);

	/* converts *end_ptr to string and places in dgr_endtime_text window */

	pasteFloatToWidget(*end_ptr, 6, *text_widget, (XmTextPosition) 0);
	XmUpdateDisplay(*text_widget);

} /* end continueCB */

/************************  doubleCB ***************************************/
/*
 *	callback function for double button
 *		doubles endtime 
 */
/****************************************************************************/


void doubleCB(Widget widget, XtPointer client_data, XtPointer call_data)

{
	FptrWptr*	ptr;	/* my own structure 
				 *  floatptr is pointer to float
				 *  widgptr is a pointer to widget */
	float*	end_ptr;	/* pointer to ending time */
	Widget*	text_widget;	/* ending time textwidget */
        char    message[80];    /* message string for message window */

        if( client_data == NULL ) return;
	
	widget = widget;
	call_data = call_data;
	ptr = (FptrWptr*) client_data;

	end_ptr = ptr->floatptr;	/* use -> as ptr is pointer to struct */
	text_widget = ptr->widgptr;

	*end_ptr = (float)floor(*end_ptr);
        *end_ptr = *end_ptr * 2;

	/* print message to message window */

        sprintf(message, "double ending time: %f", *end_ptr);
        XmTextReplace((*widget_ptr).dgr_message_window, (XmTextPosition) 0,
                XmTextGetLastPosition((*widget_ptr).dgr_message_window),
                message);

	/* converts *end_ptr to string and places in dgr_endtime_text window */

	pasteFloatToWidget(*end_ptr, 6, *text_widget, (XmTextPosition) 0);
	XmUpdateDisplay(*text_widget);

} /* end doubleCB */



void scale10KE(Widget widget, XtPointer client_data, XtPointer call_data)

{
	float*	temp;
	
	widget = widget;
	call_data = call_data;
        if( client_data == NULL ) return;

	temp = (float*) client_data;

        *temp = 0.1;

} /* scale10KE */

void scaleDKE(Widget widget, XtPointer client_data, XtPointer call_data)

{
	float*	temp;

	widget = widget;
	call_data = call_data;
        if( client_data == NULL ) return;

	temp = (float*) client_data;

        *temp = 1.5;

} /* scale10KE */





